package cn.com.qmth.examcloud.core.oe.admin.api.controller;

import cn.com.qmth.examcloud.api.commons.security.bean.User;
import cn.com.qmth.examcloud.commons.util.JsonMapper;
import cn.com.qmth.examcloud.core.oe.admin.base.utils.Check;
import cn.com.qmth.examcloud.core.oe.admin.base.utils.excel.ExportUtils;
import cn.com.qmth.examcloud.core.oe.admin.service.AsyncExportService;
import cn.com.qmth.examcloud.core.oe.admin.service.ExamScoreService;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.examscore.ExamScoreInfo;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.examscore.ExamScoreQuery;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.examscore.ObjectiveScoreInfo;
import cn.com.qmth.examcloud.core.oe.admin.service.util.AsyncExportConcurrentUtil;
import cn.com.qmth.examcloud.reports.commons.bean.AdminOperateReport;
import cn.com.qmth.examcloud.reports.commons.util.ReportsUtil;
import cn.com.qmth.examcloud.support.helper.IdentityNumberHelper;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import cn.com.qmth.examcloud.web.support.Naked;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * 考试分数相关接口
 *
 * @author: QMTH
 * @since: 2018/8/27
 */
@RestController
@Api(tags = "考试分数相关接口")
@RequestMapping("${$rmp.ctr.oe}/exam/score")
public class ExamScoreController extends ControllerSupport {

    @Autowired
    private AsyncExportService asyncExportService;

    @Autowired
    private ExamScoreService examScoreService;

    @PostMapping("/statistic/list")
    @ApiOperation(value = "查询“成绩统计”列表(分页)")
    public Page<ExamScoreInfo> getExamAuditList(@RequestBody ExamScoreQuery query) {
        Page<ExamScoreInfo> examScoreList = examScoreService.getExamScoreList(query);

        examScoreList.getContent().forEach(p -> {
            p.setIdentityNumber(IdentityNumberHelper.conceal(getRootOrgId(), p.getIdentityNumber()));
        });
        return examScoreList;
    }

    @Naked
    @GetMapping("/statistic/list/export")
    @ApiOperation(value = "导出“成绩统计”列表(Excel)", notes = "参数示例：query={\"pageNo\":1,\"pageSize\":10,\"examId\":123, ...}")
    public void exportExamRecordDetailList(@RequestParam String query, HttpServletResponse response) throws Exception {
        ExamScoreQuery newQuery = new JsonMapper().parseJson(query, ExamScoreQuery.class);
        Check.isNull(newQuery, "请求参数不能为空！");
        Check.isNull(newQuery.getExamId(), "请先选择考试批次！");
        List<ExamScoreInfo> examScoreList = examScoreService.exportExamScoreList(newQuery);
        ExportUtils.exportEXCEL("成绩统计列表", ExamScoreInfo.class, examScoreList, response);
    }

    @GetMapping("/statistic/list/export/async")
    @ApiOperation(value = "导出“成绩统计”列表(异步)",
            notes = "参数示例：query={\"rootOrgId\":0,\"examId\":1,\"startLimit\":1,\"endLimit\":1}")
    public void exportExamScores(@RequestParam String query) {
        User user = getAccessUser();
        asyncExportService.exportExamScoreStatistics(query, user);
        AsyncExportConcurrentUtil.checkOrStartJob();
        ReportsUtil.report(new AdminOperateReport(user.getRootOrgId(), user.getUserId(), "成绩统计-导出", "导出条件:" + query));
    }

    @ApiOperation(value = "根据examStudentId获取客观分信息")
    @GetMapping("/queryObjectiveScoreList")
    public List<ObjectiveScoreInfo> queryObjectiveScoreList(@RequestParam Long examStudentId) {
        return examScoreService.queryObjectiveScoreList(examStudentId);
    }

}
